<?php

namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\SM\Util\Procedure\CriaMnemonico;
use Teknisa\SM\Util\Procedure\FctGeraDadosProposta;
use Teknisa\SM\Util\Procedure\GeraContrato;
use Teknisa\SM\Util\Procedure\FctConversaoCliente;
use Teknisa\SM\Util\Procedure\FctCopiaVersPropAlimenticia;
use Teknisa\SM\Factory\Sm00001100LancamentoPropostaFactory as PropostaFactory;
use Teknisa\SM\Service\Sm00015004AgendamentoDeRecursoService;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Zeedhi\Framework\Session\Session;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Zeedhi\Framework\DTO\Row;
use Teknisa\SM\Util\Utilities;
use Teknisa\SM\Service\SmDocumentosService;

class Sm00001100LancamentoPropostaService {
    
    protected $connection;
    protected $propostaFactory;
    protected $environment;
    protected $newCode;
    protected $fileServerBaseUrl;
    protected $session;
    protected $smDocumentosService;
    
    public function __construct(EntityManager $entityManager, PropostaFactory $propostaFactory, Environment $environment, NewCode $newCode, $fileServerBaseUrl, Server $zhFileUploader, Session $session, Sm00015004AgendamentoDeRecursoService $agedRecPendente, SmDocumentosService $smDocumentosService) {
        $this->connection          = $entityManager->getConnection();
        $this->propostaFactory     = $propostaFactory;
        $this->environment         = $environment;
        $this->newCode             = $newCode;
        $this->fileServerBaseUrl   = $fileServerBaseUrl;
        $this->zhFileUploader      = $zhFileUploader;
        $this->session             = $session; 
        $this->agedRecPendente     = $agedRecPendente;
        $this->smDocumentosService = $smDocumentosService;
    }
    /**
     * Executa a procedure CRIA_MNEMONICO.
     * @param integer $nrorg
     * @param string  $tipo
     * @param integer $nrseq
     * 
     * @return string
     */
    public function executarProcedureCriaMnemonico($nrorg, $tipo, $nrseq) {
        $procedure = new CriaMnemonico($this->connection);
        $mnemonico = $procedure->executar($nrorg, $tipo, $nrseq);
        return $mnemonico;
    }
    /**
     * Executa a procedure FCT_GERA_DADOS_PROPOSTA.
     * 
     * @param integer $nrseqorcamento
     * @param integer $nrorg
     */
    public function executarProcedureFctGeraDadosProposta($nrseqorcamento, $nrorg) {
        $procedure = new FctGeraDadosProposta($this->connection);
        $procedure->executar($nrseqorcamento, $nrorg);
    }
    /**
     * Executa a procedure FCT_CONVERSAO_CLIENTE.
     * 
     * @param integer $nrorg
     * @param string  $operador
     * @param integer $nrparcnegocio
     */
    public function executarProcedureFctConversaoCliente($nrorg, $operador, $nrparcnegocio) {
        $procedure = new FctConversaoCliente($this->connection);
        $procedure->executar($nrorg, $operador, $nrparcnegocio);
    }
    /**
     * Executa a procedure GERA_CONTRATO.
     * 
     * @param integer $nrseqorcamento
     * @param integer $nrorg
     * 
     * @return integer
     */
    public function executarProcedureGeraContrato($nrseqorcamento, $nrorg) {
        $procedure = new GeraContrato($this->connection);
        $nrseqcontrato = $procedure->executar($nrorg, $nrseqorcamento);
        return $nrseqcontrato;
    }
    /**
     * Executa a procedure FCT_COPIA_VERS_PROPALIMENTICIA.
     * 
     * @param integer $nrseqorcamento
     * @param integer $nrorg
     * 
     * @return integer
     */
    public function executarProcedureFctCopiaVersPropostaAliment($nrseqorcamento, $tipo, $cdoperador, $nrorg) {
        $procedure = new FctCopiaVersPropAlimenticia($this->connection);
        $nrseqorcgerado = $procedure->executar($nrseqorcamento, $tipo, $cdoperador, $nrorg);
        return $nrseqorcgerado;
    }
    /**
     * Retorna os documentos da Proposta.
     * 
     * @param integer $nrseqorcamento
     * @param integer $nrorg
     */
    public function getDocumentosProposta($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        
        return $this->propostaFactory->getDocumentosProposta($params);
    }
    /**
     * Anexa nas propostas copiadas ou versionadas o documanto da proposta
    **/
    public function anexarDocumentosPropostaCopiada($nrseqorcamento, $nrorg, $cdoperador) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        
        $dataset = $this->propostaFactory->getDocumentosProposta($params);
        foreach ($dataset as $row) {
            $nomeArquivo   = $row["NMARQUIVO"];
            $tipo          = $row["DSTIPO"];
            $tamanhoBytes  = $row["VRTAMANHO"] * 1024;
            $urlFileServer = $row["DSCAMINHOARQ"];
            
            if (!empty($urlFileServer)) {
                //$conteudo = @file_get_contents($this->fileServerBaseUrl . $urlFileServer);
                $conteudo = $this->smDocumentosService->getDocumentoFileServer($this->fileServerBaseUrl . $urlFileServer, $nrorg);
                if ($conteudo !== false) {
                    $file = Utilities::uploadFileServer($this->zhFileUploader, "anexo_documento_proposta", $nomeArquivo, $conteudo, $tamanhoBytes, $tipo);
                    $caminhoNovo = $file["path"];
                    
                    $params = array(
                        "DSCAMINHOARQ"   => $caminhoNovo,
                        "NMARQUIVO"      => $nomeArquivo,
                        "VRTAMANHO"      => $tamanhoBytes,
                        "DSTIPO"         => $tipo,
                        "CDOPERULTATU"   => $cdoperador,
                        "NRORGULTATU"    => $nrorg,
                        "NRSEQORCADOC"   => $row["NRSEQORCADOC"],
                        "NRSEQORCAMENTO" => $nrseqorcamento,
                        "NRORG"          => $nrorg
                    );
                    
                    $this->propostaFactory->AnexaOrcamentoDoc($params);
                }
            }
        }
    }
     /**
     * Atualiza a Proposta ao aceitá-la.
     * 
     * @param integer $nrseqorcamento
     * @param string  $dataAceite
     * @param integer $nrorg
     */
    public function atualizarPropostaAceite($nrseqorcamento, $dataAceite, $nrorg,  $cdoperador) {
        $params = array(
            "NRSEQORCAMENTO"      => $nrseqorcamento,
            "IDSITUACAOORCAMENTO" => "ACEITO",
            "DTACEITEORCA"        => $dataAceite,
            "NRORG"               => $nrorg,
            "CDOPERULTATU"        => $cdoperador
        );
        $this->propostaFactory->atualizarPropostaAceite($params);
        //$this->atualizaDocumentoSituacao($nrseqorcamento, $nrorg, "ACEITO");
    }
    /**
     * Atualiza os documentos da proposta na tabela 
     * de acordo com asituação do orcamento
    **/
    public function atualizaDocumentoSituacao($nrseqorcamento, $nrorg, $idsituacao){
        try{
            $dadosEscopo = $this->getDadosDocumentoSituacao($nrseqorcamento, $nrorg, $idsituacao);
            foreach ($dadosEscopo as $dadoEscopo) {
                $params = array(
                    "NRSEQMODDOCFCT"      => $dadoEscopo["NRSEQMODDOCFCT"],
                    "NMARQUIVO"           => $dadoEscopo["NMARQUIVO"],
                    "DSCAMINHOARQ"        => $dadoEscopo["DSCAMINHOARQ"],
                    "DSTIPO"              => $dadoEscopo["DSTIPO"],
                    "VRTAMANHO"           => $dadoEscopo["VRTAMANHO"],
                    "IDSITUACAOORCAMENTO" => $idsituacao,
                    "NRSEQORCAMENTO"      => $nrseqorcamento,
                    "NRTIPOORCAMENTO"     => $dadoEscopo["NRTIPOORCAMENTO"],
                    "NRORG"               => $nrorg
                );
                $this->propostaFactory->atualizaDocumentoSituacao($params);
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    /**
     * Retorna os dados do documento da proposta de acordo com a situação
    **/
    public function getDadosDocumentoSituacao($nrseqorcamento, $nrorg, $idsituacao){
        $params = array(
            "IDSITUACAOORCAMENTO" => $idsituacao,
            "NRSEQORCAMENTO"      => $nrseqorcamento,
            "NRORG"               => $nrorg
        );
        return $this->propostaFactory->getDadosDocumentoSituacao($params);
    }
    /** 
     * Aceita a proposta 
     * @param nrseqorcamento - id do orcamento 
     * @param nrorg - numero da organizacao 
     * @param utilizacomposicao - utiliza integracao 
     * @param nrparcnegocio - id do parceiro de negocio 
     * @param cdoperador - codigo do operador 
     */
    public function aceitarProposta($nrseqorcamento, $nrorg, $utilizaComposicao, $nrparcnegocio, $cdoperador, $periodoExtendido, $dtFinVigContr = false){
        $dataAtual = date_create();
        $dataAtual = $dataAtual->format("d/m/Y");
        // Atualiza a Proposta
        $this->atualizarPropostaAceite($nrseqorcamento, $dataAtual, $nrorg, $cdoperador);
        //"true" se converte, "false" se não converte
        $converteProspectEmCliente = $this->verificaConverteProspectEmCliente($nrorg);
        //"true" se o prospect já for cliente, "false" se não for
        $validaCliente = $this->validaClienteProspect($nrparcnegocio, $nrorg);
        /* Se o parceiro de negócio for do tipo "PROSPECT", converte para "CLIENTE". (A validação do parametro 
           "Converter Prospect em Cliente no aceite da proposta?"" da Parametrização SM é feita dentro da procedure. ) */
        if ($converteProspectEmCliente && $validaCliente == false) {
            $this->executarProcedureFctConversaoCliente($nrorg, $cdoperador, $nrparcnegocio);
        }
        // Gera o Contrato
        $nrseqcontrato = $this->executarProcedureGeraContrato($nrseqorcamento, $nrorg);
        $this->anexarDocumentosContratosGerados($nrseqorcamento, $nrorg);
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento, 
            "NRORG"          => $nrorg
        );
        $itensDoOrcamneto = $this->propostaFactory->getItensProposta($params);
        foreach ($itensDoOrcamneto as $itemOrcamneto) {
            $ItensContrato = $this->getItemContratos($nrseqorcamento, $itemOrcamneto["CDSERVORCA"], $nrorg);
            foreach ($ItensContrato as $ItemContrato) {
                $this->agedRecPendente->atualizarAgendamentoPendente('5', $ItemContrato["NRITEMOCONTRATO"], $ItemContrato["NRSEQCONTRATO"], $nrseqorcamento, $itemOrcamneto["NRITEMORCAMENTO"], $nrorg);
            }
        }
    }
    /**
     * Atualiza a situação das propostas informadas.
     * 
     * @param string    $situacao
     * @param integer[] $nrseqorcamento
     * @param integer   $nrorg
     */
    public function atualizarSituacaoPropostas($situacao, array $nrseqorcamento, $nrorg) {
        $nrseqorcamento = implode(",", $nrseqorcamento);
        $this->propostaFactory->atualizarSituacaoPropostas($situacao, $nrseqorcamento, $nrorg);
        $this->atualizaDocumentoSituacao($nrseqorcamento, $nrorg, $situacao);
    }
    /**
     * retorna o item do contrato correpondente ao item da proposta
     **/
    public function getItemContratos($nrseqorcamento, $cdservorca, $nrorg){
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "CDSERVORCA"     => $cdservorca,
            "NRORG"          => $nrorg
        );
        return $this->propostaFactory->getItemContratos($params);
    }
    /**
     * Anexa os documentos do contratos gerados a partir da proposta
    **/
    public function anexarDocumentosContratosGerados($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG" => $nrorg
        );
        
        $dataset = $this->propostaFactory->getDocumentosContratosGerados($params);
        foreach ($dataset as $row) {
            $nomeArquivo   = $row["NMARQUIVO"];
            $tipo          = $row["DSTIPO"];
            $tamanhoBytes  = $row["VRTAMANHO"] * 1024;
            $urlFileServer = $row["DSCAMINHOARQ"];
            
            if (!empty($urlFileServer)) {
                //$conteudo = @file_get_contents($this->fileServerBaseUrl . $urlFileServer);
                $conteudo = $this->smDocumentosService->getDocumentoFileServer($this->fileServerBaseUrl . $urlFileServer, $nrorg);
                if ($conteudo !== false) {
                    $file = Utilities::uploadFileServer($this->zhFileUploader, "anexo_documento_contrato", $nomeArquivo, $conteudo, $tamanhoBytes, $tipo);
                    $caminhoNovo = $file["path"];
                    
                    $params = array(
                        "DSCAMINHOARQ"  => $caminhoNovo,
                        "NRSEQCONTDOC"  => $row["NRSEQCONTDOC"],
                        "NRSEQCONTRATO" => $row["NRSEQCONTRATO"],
                        "NRORG"         => $nrorg
                    );
                    
                    $this->propostaFactory->atualizarCaminhoDocumentoContrato($params);
                }
            }
        }
    }
    /**
     * Retorna "true" se converte, "false" se não converte
     **/
    public function verificaConverteProspectEmCliente($nrorg) {
        $params = array(
            "NRORG" => $nrorg
        );
        $dataset = $this->propostaFactory->getIdConverteProspectEmCliente($params);
        return ($dataset[0]["IDCONVCLIENTE"] == "S");
    }
    /**
     * Retorna "true" se o prospect já for cliente, "false" se não for
     **/
    public function validaClienteProspect($nrparcnegocio, $nrorg) {
        $params = array(
            "NRPARCNEGOCIO" => $nrparcnegocio,
            "NRORG"         => $nrorg
        );
        $dataset = $this->propostaFactory->getCountClienteProspect($params);
        return ((int)$dataset[0]["QTDE"] > 0);
    }
    /**
     * Retorna o toal do orcamento sem markup
    **/
    public function getTotalSemMarkup(array $params){
        $mObras       = $this->propostaFactory->getVrMObraProposta($params);
        $invetimentos = $this->propostaFactory->getVrInvestProposta($params);
        $taxas        = $this->propostaFactory->getVrTaxasProposta($params);
        $despesas     = $this->propostaFactory->getVrDespesasProposta($params);
        $servicos     = $this->propostaFactory->getVrServicosProposta($params);

        $totalOrc   = $mObras + $invetimentos + $despesas + $servicos;
        $totalStaxa = $totalOrc;
        $totalOrc   *= (1 + ($taxas/100));
        return $totalOrc;
    }
    /**
     * Calcula o valor total do orcamento
    **/
    public function calcularValorOrcamento(array $params){
        $taxas        = $this->propostaFactory->getVrTaxasProposta($params);
        $mObras       = $this->propostaFactory->getVrMObraProposta($params);
        $invetimentos = $this->propostaFactory->getVrInvestProposta($params);
        $impostos     = $this->propostaFactory->getVrImpostosProposta($params);
        $despesas     = $this->propostaFactory->getVrDespesasProposta($params);
        $servicos     = $this->propostaFactory->getVrServicosProposta($params);
        
        $markups      = $this->propostaFactory->getVrMarkup($params);
        $markupAplic  = $markups["VRPERCMARKUPAPLICADO"];
        $markupIdeal  = $markups["VRPERCMARKUP"];
        
        $totalOrc   = $mObras + $invetimentos + $despesas + $servicos;
        $totalStaxa = $totalOrc;
        
        $totalOrc *= (1 + ($taxas/100));
        $totalCtaxa = $totalOrc;
        
        $totalOrcIdeal = $totalOrc;
        $totalOrcIdeal *= (1 + ($markupIdeal/100));
        $totalOrc      *= (1 + ($markupAplic/100));
        
        $datasetFin = array(
            "TOTALBRUTO"    => $totalOrc,
            "IMPOSTO"       => ($totalCtaxa * (1 + ($markupAplic/100))) * ($impostos/100),
            "TOTALLIQ"      => $totalOrc - ($totalOrc * ($impostos/100)),
            "CALIMENTAR"    => $servicos,
            "INVESTIMENTOS" => $invetimentos,
            "DESPESAS"      => $despesas,
            "TAXAS"         => $totalStaxa * ($taxas/100),
            "MARKUP"        => $totalCtaxa * ($markupAplic/100),
            "MOBRA"         => $mObras,
            "TOTALCTAXA"    => $totalCtaxa,
            "TOTALSTAXA"    => $totalStaxa,
            "TOTALIDEAL"    => $totalOrcIdeal,
            "IMPOSTOIDEAL"  => $totalOrcIdeal * ($impostos/100),
            "TOTALLIQIDEAL" => $totalOrcIdeal - ($totalOrcIdeal * ($impostos/100)),
            "MARKUPIDEAL"   => $totalCtaxa * ($markupIdeal/100)
        );
        
        return $datasetFin;
    }
    /**
     * Calcula o valor total de cada serviço do orcamento
    **/
    public function calculaValoresServicos($totalOrca, $totalIdeal, $params){
    	$arrServ = array();
    	$dadosServ = $this->propostaFactory->getInfoServicosProp($params);
    	foreach($dadosServ as $dadoServ){
    	    $qttotal = $dadoServ["QTTOTALORCA"] <= 0 ? 1 : $dadoServ["QTTOTALORCA"];
    		$servico = array(
    			"NRCOMPSERVICO"       => $dadoServ["NRCOMPSERVICO"],
                "CDSERVORCA"          => explode("_", $dadoServ["CDSERVORCA"]),
                "NMELEMENTO"          => $dadoServ["NMELEMENTO"],
                "VRCUSTO"             => $dadoServ["VRCUSTOSERV"],
                "VRPERCSERVICO"       => $dadoServ["VRPERCSERVICO"],
                "VRPRECOIDEAL"        => $dadoServ["VRCUSTOIDEAL"],
                "VRUNITITEMORCA"      => $dadoServ["VRCUSTOINFORMADO"],
                "VRUNITITEMORCA_GRID" => $dadoServ["VRCUSTOINFORMADO"],
                "VRTOTALITEMORCA"     => $dadoServ["VRTOTSERV"],
                "VRTOTIDEALSERV"      => ($totalIdeal/$qttotal) * $dadoServ["QTTOTALSERV"],
                "QTITEMORCAMENTO"     => $dadoServ["QTTOTALSERV"],
                "NRTIPOORCAMENTO"     => explode("_", $dadoServ["NRTIPOORCAMENTO"])
    		);
    
    		array_push($arrServ, $servico);
    	}
    
    	return $arrServ;
    }
    /**
     * Retorna as informações do resultafo das propostas já inseridas
    **/
    public function getInfoResultado($nrseqorcamento, $nrorg){
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        
        $financeiro = $this->calcularValorOrcamento($params);
        $servicos   = $this->calculaValoresServicos($financeiro["TOTALBRUTO"], $financeiro["TOTALIDEAL"], $params);
        $resultado  = array(
            "SERVICOS"   => $servicos,
            "FINANCEIRO" => $financeiro
        );
        
        return $resultado;
    }
    /**
     * Anexa o documento ligado ao modelo de proposta 
     * aos dados da proposta
     */
    public function anexaDocumento($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        $anexos = $this->propostaFactory->buscaDocAnexado($params);
        foreach ($anexos as $anexo) {
            $nrseqorcadoc = $anexo["NRSEQORCADOC"];
            $nome         = $anexo["NMARQUIVO"];
            $caminho      = $anexo["DSCAMINHOARQ"];
            $tipo         = $anexo["DSTIPO"];
            $tamanho      = $anexo["VRTAMANHO"];
            if (!empty($caminho)) {
                //$arquivoDoc = @file_get_contents($this->fileServerBaseUrl . $caminho);
                $arquivoDoc = $this->smDocumentosService->getDocumentoFileServer($this->fileServerBaseUrl . $caminho, $nrorg);
                if ($arquivoDoc !== false) {
                    $arquivoDoc = base64_encode($arquivoDoc);
                    /* 2 - Deleta Anexo
                    $this->zhFileUploader->deleteFile($caminho); */
                    /* 3 - Anexar o documento novamente */ 
                    $anexoNovo = $this->anexaOrcamentoDoc($arquivoDoc,  $nome, $tipo, $tamanho);
                    $nomeNovo    = $anexoNovo["name"];
                    $caminhoNovo = $anexoNovo["path"];
                    $tamanhoNovo = $anexoNovo["size"];
                    $tamanhoNovo = $tamanhoNovo / 1024;
                    $tipoNovo    = $anexoNovo["type"];
                    /* 4 - Atualiza ORCAMENTODOC */
                    $params = array(
                        "NMARQUIVO"      => $nomeNovo,
                        "DSCAMINHOARQ"   => $caminhoNovo,
                        "VRTAMANHO"      => $tamanhoNovo,
                        "DSTIPO"         => $tipoNovo,
                        "CDOPERULTATU"   => $this->environment->getCdOperador(),
                        "NRORGULTATU"    => $nrorg,
                        "NRSEQORCADOC"   => $nrseqorcadoc,
                        "NRSEQORCAMENTO" => $nrseqorcamento,
                        "NRORG"          => $nrorg
                    );
                    $anexos = $this->propostaFactory->AnexaOrcamentoDoc($params);
                }
            }
        }
    }
    /**
    * Anexa o novo documento do orçamento 
    **/
    public function anexaOrcamentoDoc($arquivoDoc, $nome, $tipo, $tamanho){
         $fileData = array(
            'name'    => $nome,
            'size'    => $tamanho * 1024,
            'type'    => $tipo,
            'b64File' => 'data:'.$tipo.';base64,'.$arquivoDoc,
            'webkitRelativePath' => ''
        );

        $row = new Row(array('FILE' => array($fileData)));
        $this->zhFileUploader->uploadFilesInRow($row, 'FILE', 'anexo_documento_proposta');
        return $row["FILE"][0];
    }
    /**
     * Exclui os escopos da proposta.
     */
    public function excluirEscopos($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        $this->propostaFactory->excluirEscopos($params);
    }
    
    /**
     * A função "buscaModeloDocumento" busca o modelo de documentos para cada escopo 
     * a fim de ser inserido na tabela FCT_RELACESCOPOPROPOSTA
     **/
    public function buscaModeloDocumento($modelo, $escopo, $nrorg) {
        $params = array(
            "NRORCAMENTOMODELO" => $modelo,
            "NRTIPOORCAMENTO"   => $escopo,
            "NRORG"             => $nrorg
        );
        $dataset = $this->propostaFactory->buscaModeloDocumento($params);
        return $dataset;
    }
    /**
     * A função "buscaFilialHierarquia" busca a filial vinculado ao 
     * operador na hierarquia de operadores (CRM_HIERARQUIAOPER)
     * a fim de ser inserido na tabela FCT_RELACESCOPOPROPOSTA
     **/
    public function buscaFilialHierarquia($operador, $nrorg) {
        $params = array(
            "CDOPERADOR" => $operador,
            "NRORG"      => $nrorg
        );
        $dataset = $this->propostaFactory->buscaFilialHierarquia($params);
        return $dataset;
    }
    /**
     * Insere todas as informações de serviço, mão de obra, despesa, investimento, taxa, imposto
     * da proposta
    **/
    public function inserirTudoProposta($itens, $custos, $servicos, $mObras, $despesas, $investimentos, $taxas, $nrseqorcamento, $nrorg, $cdoperador, $nrtipoorcamento, $impostos, $servicoRelac){
        $this->connection->beginTransaction();
        try{
            $elementos = $this->session->get("inserir_elementos_proposta");
            if(is_array($elementos)) { 
                $elementos = $this->validarQuantidadesElementos($custos, $elementos, $servicos);
                $this->inserirServicosProposta($elementos, $cdoperador, $nrorg, $nrseqorcamento);
            }
            
            $this->inserirImpostoProposta($impostos, $nrseqorcamento, $nrorg, $cdoperador);
            $this->inserirDespesasProposta($despesas, $cdoperador, $nrorg, $nrseqorcamento);
            $this->inserirInvestimentosProposta($investimentos, $cdoperador, $nrorg, $nrseqorcamento);
            $this->inserirTaxasProposta($taxas, $cdoperador, $nrorg, $nrseqorcamento);
            
            $this->inserirMaoObrasProposta($mObras, $cdoperador, $nrorg, $nrseqorcamento);
            $this->inserirResultadoOrcamento($itens, $cdoperador, $nrorg, $nrseqorcamento);
            
            $itensElemento = $this->getItensTipoElementoProposta($nrseqorcamento, $nrorg);
            $nrItensPai    = $this->validarParaInserirItensProposta($itens, $cdoperador, $nrorg, $nrseqorcamento, $nrtipoorcamento, $servicoRelac, $itensElemento);
            $this->inserirItenTipoElemento($itensElemento, $nrItensPai, $nrseqorcamento, $nrtipoorcamento, $nrorg, $cdoperador);
            $this->atualizarValorProposta($nrseqorcamento, $nrorg);
            
            $this->session->set("inserir_elementos_proposta", "");
            $this->connection->commit();
        } catch(\Exception $e){
            $this->connection->rollback();
            $this->excluirEscopos($nrseqorcamento, $nrorg);
            $this->excluirProposta($nrseqorcamento, $nrorg);
            throw $e;
        }
    }
    /**
     * Busca os itens de tipo elemento
    **/
    public function getItensTipoElementoProposta($nrseqorcamento, $nrorg){
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        return $this->propostaFactory->getItensTipoElementoProposta($params);
    }
    /**
     * altera os valores dos elementos de acordo com o que for 
     * informado na tela
    **/
    public function validarQuantidadesElementos($custos, $elementos, $servicos){
        try{
            if(count($elementos) > 0){
                foreach ($elementos as &$elemento) {
                    foreach($custos as $custo){
                        if($elemento["CDSERVORCARELAC"] == $custo["CDSERVORCARELAC"] && $elemento["IDELEMENTORAIZ"] == $custo["IDELEMENTORAIZ"]){
                            $elemento["VRCUSTOSERV"] = (float)$custo["VRCUSTOSERV"];
                            break;
                        }
                    }
                    
                    foreach($servicos as $servico){
                        if($elemento["NRCOMPSERVICO"] == $servico["NRCOMPSERVICO"]){
                            $elemento["QTSEGUNDA"]    = (int)$servico["QTSEGUNDA"];
                            $elemento["QTTERCA"]      = (int)$servico["QTTERCA"];
                            $elemento["QTQUARTA"]     = (int)$servico["QTQUARTA"];
                            $elemento["QTQUINTA"]     = (int)$servico["QTQUINTA"];
                            $elemento["QTSEXTA"]      = (int)$servico["QTSEXTA"];
                            $elemento["QTSABADO"]     = (int)$servico["QTSABADO"];
                            $elemento["QTDOMINGO"]    = (int)$servico["QTDOMINGO"];
                            $elemento["QTMENSAL"]     = (int)$servico["QTMENSAL"];
                            $elemento["HRENTRADA"]    = $servico["HRENTRADA"];
                            $elemento["HRSAIDA"]      = $servico["HRSAIDA"];
                            break;
                        }
                    }
                }
            }
            return $elementos;
        } catch(\Exception $e){
            throw $e;
        }
    }
    /**
     * Atualiza o valor total da proposta com a soma dos itens.
     */
    public function atualizarValorProposta($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        
        $valor = $this->propostaFactory->getSomaValorItensProposta($params);
        $valor = $valor[0]["VRTOTALITEMORCA"];
        $params["VRORCAMENTO"] = $valor;
        $this->propostaFactory->atualizarValorProposta($params);
        
        return $valor;
    }
    /**
     * Exclui uma proposta.
     **/
    public function excluirProposta($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        $this->propostaFactory->excluirProposta($params);
    }
    /**
     * Salva os serviços ligados a uma proposta
    **/
    public function inserirServicosProposta($elementos, $cdoperador, $nrorg, $nrseqorcamento){
        try {
            foreach($elementos as $elemento){
                $nrrelacservorca = $this->newCode->getCodeByTableName("FCT_RELACSERVORCAMENTO", $nrorg, 1, 10, true);
                $params = array(
                    "NRRELACSERVORCA" => $nrrelacservorca,
                    "NRSEQORCAMENTO"  => $nrseqorcamento,
                    "IDESCOLHIDO"     => 'S',
                    "IDTIPOELEMENTO"  => $elemento["IDTIPOELEMENTO"],
                    "CDSERVORCA"      => $elemento["CDSERVORCARELAC"],
                    "NRCOMPSERVICO"   => $elemento["NRCOMPSERVICO"],
                    "VRCUSTOSERV"     => (float)$elemento["VRCUSTOSERV"],
                    "VRTOTMENSALSERV" => (float)$elemento["VRCUSTOSERV"] * $elemento["QTMENSAL"],
                    "QTSEGUNDA"       => (int)$elemento["QTSEGUNDA"],
                    "QTTERCA"         => (int)$elemento["QTTERCA"],
                    "QTQUARTA"        => (int)$elemento["QTQUARTA"],
                    "QTQUINTA"        => (int)$elemento["QTQUINTA"],
                    "QTSEXTA"         => (int)$elemento["QTSEXTA"],
                    "QTSABADO"        => (int)$elemento["QTSABADO"],
                    "QTDOMINGO"       => (int)$elemento["QTDOMINGO"],
                    "QTMENSAL"        => (int)$elemento["QTMENSAL"],
                    "CDTPMEDIDASERV"  => null, //$elemento["CDTPMEDIDASERV"],
                    "QTMEDSERVORCA"   => 0, //$elemento["QTMEDSERVORCA"],
                    "IDSERVOBRIGCOMP" => 'S',//$elemento["IDSERVOBRIGCOMP"],
                    "IDTPDESCORCA"    => "N", //$elemento["IDTPDESCORCA"],
                    "VRPERCDESCORCA"  => 0, //$elemento["VRPERCDESCORCA"],
                    "VRTOTMSLSAJUSTE" => (float)$elemento["VRCUSTOSERV"] * (float)$elemento["QTMENSAL"],
                    "IDELEMENTO"      => $elemento["IDELEMENTO"],
                    "IDELEMENTOPAI"   => $elemento["IDELEMENTOPAI"],
                    "NRTIPOORCAMENTO" => $elemento["NRTIPOORCAMENTO"],
                    "NRORG"           => $nrorg,
                    "NRORGINCLUSAO"   => $nrorg,
                    "CDOPERINCLUSAO"  => $cdoperador,
                    "NRORGULTATU"     => $nrorg,
                    "CDOPERULTATU"    => $cdoperador,
                    "IDATIVO"         => 'S'
                );
                $this->propostaFactory->inserirServicosProposta($params);
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    /**
     * Insere a mão de obra ligada proposta
     * se o item for de formação auxiliar irá inserir -1 no nritemorcamento
     * caso contrário irá inserir 0
    **/
    public function inserirMaoObrasProposta($mObras, $cdoperador, $nrorg, $nrseqorcamento){
        try{
            $relacformacaoprop = array();
            foreach ($mObras as $mObra) {
                $nrformacaoprop = $this->newCode->getCodeByTableName("FCT_RELACFORMACAOPROP", $nrorg, 1, 10, true);
                $params = array(
                    "NRRELACFORMACAOPROP" => $nrformacaoprop,
                    "NRSEQORCAMENTO"      => $nrseqorcamento,
                    "NRFORMPRECOSERVICO"  => $mObra["NRFORMPRECOSERVICO"],
                    "NRITEMORCAMENTO"     => -1,
                    "QTFORMACAOPROP"      => $mObra["QTFORMACAO"],
                    "CDSERVORCARELAC"     => $mObra["CDSERVORCA"],
                    "NRORG"               => $nrorg,
                    "IDATIVO"             => 'S',
                    "NRORGINCLUSAO"       => $nrorg,
                    "CDOPERINCLUSAO"      => $cdoperador,
                    "NRORGULTATU"         => $nrorg,
                    "CDOPERULTATU"        => $cdoperador
                );
                $this->propostaFactory->inserirMaoObrasProposta($params);
                $this->updateItensFormacaoProposta($mObra, $nrorg, $nrseqorcamento, $nrformacaoprop);
            }
            return $relacformacaoprop;
        } catch(\Exception $e){
            throw $e;
        }
    }
    /**
     * Insere as despesas da proposta
    **/
    public function inserirDespesasProposta($despesas, $cdoperador, $nrorg, $nrseqorcamento){
        try{
            foreach ($despesas as $despesa) {
                $nrdespprop = $this->newCode->getCodeByTableName("FCT_RELACDESPPROPOSTA", $nrorg, 1, 10, true);
                $params = array(
                    "NRDESPPROP"        => $nrdespprop,
                    "NRSEQORCAMENTO"    => $nrseqorcamento,
                    "CDDESPESA"         => $despesa["CDDESPESA"],
                    "VRDESPESAORCA"     => $despesa["VRTOTDESPPERIODO"],
                    "CDPERIODODESPPROP" => $despesa["CDPERIODO"],
                    "NRORG"             => $nrorg,
                    "IDATIVO"           => 'S',
                    "NRORGINCLUSAO"     => $nrorg,
                    "CDOPERINCLUSAO"    => $cdoperador,
                    "NRORGULTATU"       => $nrorg,
                    "CDOPERULTATU"      => $cdoperador,
                    "VRUNITDESPESA"     => $despesa["VRDESPESA"]
                );
                
                $this->propostaFactory->inserirDespesasProposta($params);
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    /**
     * Insere as taxas da proposta
    **/
    public function inserirTaxasProposta($taxas, $cdoperador, $nrorg, $nrseqorcamento){
        try{
            foreach ($taxas as $taxa) {
                $nrtaxaprop = $this->newCode->getCodeByTableName("FCT_RELACTAXAPROPOSTA", $nrorg, 1, 10, true);
                $params = array(
                    "NRTAXAPROP"     => $nrtaxaprop,
                    "NRSEQORCAMENTO" => $nrseqorcamento,
                    "CDTAXA"         => $taxa["CDTAXA"],
                    "VRPERCTAXAORC"  => $taxa["VRPERCTAXA"],
                    "NRORG"          => $nrorg,
                    "IDATIVO"        => 'S',
                    "NRORGINCLUSAO"  => $nrorg,
                    "CDOPERINCLUSAO" => $cdoperador,
                    "NRORGULTATU"    => $nrorg,
                    "CDOPERULTATU"   => $cdoperador
                );
                
                $this->propostaFactory->inserirTaxasProposta($params);
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    /**
     * Insere os investimnentos da proposta
    **/
    public function inserirInvestimentosProposta($investimentos, $cdoperador, $nrorg, $nrseqorcamento){
        try{
            foreach ($investimentos as $investimento) {
                $nrinvestprop = $this->newCode->getCodeByTableName("FCT_RELACINVESTPROPOSTA", $nrorg, 1, 10, true);
                $params = array(
                    "NRINVESTPROP"     => $nrinvestprop,
                    "NRSEQORCAMENTO"   => $nrseqorcamento,
                    "NRINVESTIMENTO"   => $investimento["NRINVESTIMENTO"],
                    "VRUNITINVESTORCA" => $investimento["VRINVESTIMENTO"],
                    "QTINVESTORCA"     => $investimento["QTINVESTIMENTO"],
                    "VRTOTINVETORCA"   => $investimento["VRINVESTIMENTO"] * $investimento["QTINVESTIMENTO"],
                    "QTMESARMOTIZACAO" => $investimento["QTMESARMOTIZACAO"],
                    "VRPARCELA"        => $investimento["VRPARCELA"],
                    "NRORG"            => $nrorg,
                    "IDATIVO"          => 'S',
                    "NRORGINCLUSAO"    => $nrorg,
                    "CDOPERINCLUSAO"   => $cdoperador,
                    "NRORGULTATU"      => $nrorg,
                    "CDOPERULTATU"     => $cdoperador
                );
                
                $this->propostaFactory->inserirInvestimentosProposta($params);
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    /**
     * Insere os itens da proposta
    **/
    public function validarParaInserirItensProposta($itens, $cdoperador, $nrorg, $nrseqorcamento, $nrtipoorcamento, $servicoRelac, $itensElemento){
        $cdservrelac   = array();
        $nritemorcapai = array();
        foreach($servicoRelac as $serv){
            array_push($cdservrelac, $serv["CDSERVORCARELAC"]);
        }
        
        $cdservrelac = array_unique($cdservrelac);
        if(count($cdservrelac) == 0)
            throw new \Exception("Operação bloqueada!<br><br>Não é possível inserir propostas de alimentação sem itens. Certifique-se que há serviços ligados a ela.");
            
        foreach ($cdservrelac as $cdservorca) {
            $vrtotal = 0;
            foreach($servicoRelac as $serv){
                foreach($itens as $item){
                    if((int)$cdservorca == $serv["CDSERVORCARELAC"] && in_array($serv["CDSERVORCARELAC"], $item["CDSERVORCA"]))
                        $vrtotal = $vrtotal + ($item["VRTOTALITEMORCA"] * ($serv["VRPERCSERVICO"]/100));
                }
            }
            
            foreach($itensElemento as $elemento){
                if((int)$cdservorca == $elemento["CDSERVORCA"]) {
                    $nrtipoorcamento = $elemento["NRTIPOORCAMENTO"];
                    break;
                }
            }
            
            $dadosServico    = $this->getDadosServicoRelac((int)$cdservorca, $nrorg, $nrtipoorcamento, explode("_", $item["NRCOMPSERVICO"]));
            $nmelemento      = !empty($dadosServico) ? $dadosServico[0]["NMSERVORCA"] : $item["NMELEMENTO"];
            $nrtipoorcamento = !empty($dadosServico) ? $dadosServico[0]["NRTIPOORCAMENTO"] : null;
            $nritemorcamento = $this->salvarItens($nrseqorcamento, $nmelemento, $nrtipoorcamento, $vrtotal, $cdservorca, null, $nrorg, $cdoperador, null, null, 1);
            $paramPai = array(
                "NRITEMORCAMENTO" => $nritemorcamento,
                "CDSERVORCA"      => $cdservorca
            );
            array_push($nritemorcapai, $paramPai);
        }
        return $nritemorcapai;
    }
    /**
     * Busca o nritem pai e manda salvar um item do tipo elemento
    **/
    public function inserirItenTipoElemento($itensElemento, $nrItensPai, $nrseqorcamento, $nrtipoorcamento, $nrorg, $cdoperador){
        foreach($itensElemento as $item){
            foreach($nrItensPai as $nritempai){
                if($item["CDSERVORCA"] == $nritempai["CDSERVORCA"]){
                    $this->salvarItens($nrseqorcamento, $item["NMELEMENTOPAI"], $item["NRTIPOORCAMENTO"], $item["VRCUSTOSERV"], $item["CDSERVORCA"], $nritempai["NRITEMORCAMENTO"], $nrorg, $cdoperador, $item["IDELEMENTOPAI"], $item["CDTPMEDIDASERV"], $item["QTMENSAL"]);
                }
            }
        }
    }
    /**
     * Insere um item da proposta
    **/
    public function salvarItens($nrseqorcamento, $nmelemento, $nrtipoorcamento, $vrunitario, $cdservorca, $nritemorcapai, $nrorg, $cdoperador, $idelemento, $cdtpmedida, $quantidade){
        $nritemorcamento = $this->newCode->getCodeByTableName("NRITEMORCAMENTO", $nrorg, 1, 10, true);
        $params = array(
            "NRITEMORCAMENTO"       => $nritemorcamento,
            "NRSEQORCAMENTO"        => $nrseqorcamento,
            "CDSERVORCA"            => (int)$cdservorca,
            "NMITEMORCAMENTO"       => $nmelemento,
            "DSQTITEMORCA"          => "Referente ao Produto",
            "QTITEMORCAMENTO"       => $quantidade,
            "NRITEMORCAMENTOPAI"    => $nritemorcapai,
            "VRUNITITEMORCA"        => $vrunitario,
            "VRTOTALITEMORCA"       => $vrunitario * $quantidade,
            "VRITEMSEMDESC"         => $vrunitario * $quantidade,
            "CDTPMEDIDASERV"        => null,
            "QTMEDIDAORCA"          => null,
            "IDITEMOBRIGORCA"       => "N",
            "IDSOMASUBITENS"        => "N",
            "IDSOMANOITEMPAI"       => "S",
            "IDSOMANOTOTALORCA"     => "S",
            "IDIMPRESSAOORCA"       => "S",
            "IDPRECOLOCACAO"        => "PROPORCIONAL_TEMPO",
            "IDMULTMEDVRTOT"        => "S",
            "IDBLOQALTORC"          => "N",
            "IDADICITEMORC"         => "N",
            "IDADICCOBRITEMORC"     => "N",
            "IDTPDESCORCA"          => 'N',
            "IDTRANSPORTADORA"      => "PROPRIA",
            "IDATIVO"               => "S",
            "NRORG"                 => $nrorg,
            "CDOPERINCLUSAO"        => $cdoperador,
            "NRORGINCLUSAO"         => $nrorg,
            "NRTIPOORCAMENTO"       => $nrtipoorcamento,
            "IDFATURAORCA"          => null,
            "CDMODPGTO"             => null,
            "NRGRUPOFATURA"         => null,
            "QTPARCELASFATU"        => null,
            "NRDIAVENCPARCELASFATU" => null,
            "VRPERCDESCORCA"        => 0,
            "NMREDUITEMORCA"        => (string)$nritemorcamento,
            "IDELEMENTO"            => $idelemento,
            "CDTPMEDIDAVARIAVEL"    => $cdtpmedida
        );
        $this->propostaFactory->inserirItensProposta($params);
        return $nritemorcamento;
    }
    /**
     * Insere as informações finais dos valores de cada serviço final
     * da proposta - tela de resultado -> aba serviços
    **/
    public function inserirResultadoOrcamento($itens, $cdoperador, $nrorg, $nrseqorcamento){
        foreach($itens as $item){
            $nrresultorca = $this->newCode->getCodeByTableName("FCT_RESULTORCALIMENT", $nrorg, 1, 10, true);
            $params = array(
                "NRRESULTORCA"      => $nrresultorca,
                "NRSEQORCAMENTO"    => $nrseqorcamento,
                "VRCUSTOINFORMADO"  => $item["VRUNITITEMORCA"],
                "VRCUSTOIDEAL"      => $item["VRPRECOIDEAL"],
                "IDELEMENTORAIZ"    => $item["IDELEMENTO"],
                "NRCOMPSERVICORAIZ" => $item["NRCOMPSERVICO"],
                "QTTOTALSERV"       => $item["QTITEMORCAMENTO"],
                "VRTOTSERV"         => $item["VRTOTALITEMORCA"],
                "VRPERCSERVICO"     => $item["VRPERCSERVICO"],
                "IDATIVO"           => "S",
                "NRORG"             => $nrorg,
                "CDOPERINCLUSAO"    => $cdoperador,
                "NRORGINCLUSAO"     => $nrorg
            );
            $this->propostaFactory->inserirResultadoOrcamento($params);
        }
    }
    /**
     * Retorna o nome e o escopo do serviço ligado ao item da proposta
    **/
    public function getDadosServicoRelac($cdservorca, $nrorg, $nrtipoorcamento, $nrcomposicao){
        if(count($nrtipoorcamento) > 1)
            $nrtipoorcamento = implode(",", $nrtipoorcamento);
        $nrcomposicao    = implode(",", $nrcomposicao);
        $params = array(
            "CDSERVORCA" => $cdservorca,
            "NRORG"      => $nrorg
        );
        return $this->propostaFactory->getDadosServicoRelac($params, $nrtipoorcamento, $nrcomposicao);
    }
    /**
     * Altera o NRSEQORCAMENTO dos itens de formação de preço
     * ligados a proposta inserida
    **/
    public function updateItensFormacaoProposta($mObra, $nrorg, $nrseqorcamento, $nrrelacform){
        $params = array(
            "NRORG"               => $nrorg,
            "NRSEQORCAMENTO"      => $nrseqorcamento,
            "NRRELACFORMACAOPROP" => $nrrelacform,
            "NRFORMPRECOSERVICO"  => $mObra["NRFORMPRECOSERVICO"],
            "IDFORMACAO"          => $mObra["IDFORMACAO"]
        );
        
        $this->propostaFactory->updateItensFormacaoProposta($params, $params);
    }
    /**
     * Retorna as informações do serviço para a widget de custo do serviço
    **/
    public function getComposicoesServicos($nrtipoorcamento, $nrorg, $nrcomposicao){
        $nrtipoorcamento = implode(",", $nrtipoorcamento);
        $nrcomposicao    = implode(",", array_unique($nrcomposicao));
        $elementos = $this->propostaFactory->getElementosComposicaoServico($nrtipoorcamento, $nrorg, $nrcomposicao);
        $this->session->set("inserir_elementos_proposta", $elementos);
        
        $elementosTela = $this->propostaFactory->getElementosComposicaoServicoSemNrComposicao($nrtipoorcamento, $nrorg, $nrcomposicao);
        $elementosTela = $this->validarExistenciaElementoRaiz($elementosTela);
        return $elementosTela;
    }
    /**
     * Valida e agrupa os elementos de mesma composição
    **/
    public function validarExistenciaElementoRaiz($elementos){
        $arrElemenstosCriados = array();
        foreach($elementos as $key=>&$elemento){
            if($elemento["IDELEMENTOPAI"] !== null && !in_array($elemento["IDELEMENTOPAI"], $arrElemenstosCriados)){
                $existe = false;
                $count  = 0;
                foreach ($elementos as $eleRaiz) {
                    if($elemento["IDELEMENTO"] !== $eleRaiz["IDELEMENTO"] && $elemento["IDELEMENTOPAI"] == $eleRaiz["IDELEMENTO"]){
                        if($count < 2){
                            $count = $count + 1;
                        } else { 
                            $existe = true;
                            break;
                        }
                    }
                }
                
                if(!$existe){
                    $elementoPai = $elemento;
                    $elementoPai["IDELEMENTO"]      = $elemento["IDELEMENTOPAI"];
                    $elementoPai["NMSERVORCARELAC"] = "";
                    $elementoPai["CDSERVORCARELAC"] = "";
                    $elementoPai["IDELEMENTOPAI"]   = null;
                    $elementos[] = $elementoPai;
                    array_push($arrElemenstosCriados, $elemento["IDELEMENTOPAI"]);
                }
            }
        }
        return $elementos;
    }
    /**
     * Retorna para a tela os dados dos serviços ligados ao escopo e modelo
    public function getServicosPropostaInserindo($nrtipoorcamento, $nrorg, $nrmodelo){
        $nrtipoorcamento = implode(",", $nrtipoorcamento);
        return $this->propostaFactory->getServicosPropostaInserindo($nrtipoorcamento, $nrorg, $nrmodelo);
    }
    **/
    /**
     * Insere os impostos ligados ao orçamento criando uma relação entre eles
    **/
    public function inserirImpostoProposta($impostos, $nrseqorcamento, $nrorg, $cdoperador){
        foreach($impostos as $imposto){
            $nrimporca = $this->newCode->getCodeByTableName("FCT_RELACIMPOSTOORCA", $nrorg, 1, 10, true);
            $params = array(
                "NRIMPOSTOORCA"  => $nrimporca,
                "NRSEQORCAMENTO" => $nrseqorcamento,
                "CDIMPOSTO"      => $imposto["CDIMPOSTO"],
                "VRPERCIMPOSTO"  => $imposto["VRPRECIMPOSTO"],
                "NRORG"          => $nrorg,
                "IDATIVO"        => 'S',
                "NRORGINCLUSAO"  => $nrorg,
                "CDOPERINCLUSAO" => $cdoperador,
                "NRORGULTATU"    => $nrorg,
                "CDOPERULTATU"   => $cdoperador
            );
            
            $this->propostaFactory->inserirImpostoProposta($params);
        }
    }
    /**
     * Efetua o cálculo e manda alterar o valor do item após a alteração de um 
     * componente da proposta
    **/
    public function atualizarValorItem($nrseqorcamento, $nrorg, $cdoperador){
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento, 
            "NRORG"          => $nrorg
        );
        $resultado   = $this->calcularValorOrcamento($params);
        $elementos   = $this->propostaFactory->getInfoElementosServicosProp($params);

        foreach ($elementos as $elemento) 
            $this->alterarResultadoProposta($resultado["TOTALBRUTO"], $nrseqorcamento, $elemento["NRCOMPSERVICO"], $nrorg, $elemento["QTMENSAL"], $cdoperador, $resultado["TOTALIDEAL"]);
        
        $this->updateItemOrcamento($nrseqorcamento, $nrorg, $resultado["TOTALBRUTO"], $cdoperador);
    }
    /**
     * Retorna a quantidade e o valor total dos elementos 
     * de uma proposta
    **/
    public function alterarResultadoProposta($vrtotal, $nrseqorcamento, $nrcomposicao, $nrorg, $qtservico, $cdoperador, $totalIdeal){
        $params = array(
            "NRSEQORCAMENTO"    => $nrseqorcamento,
            "NRORG"             => $nrorg,
            "VRTOTAL"           => $vrtotal,
            "NRCOMPSERVICORAIZ" => $nrcomposicao,
            "QTTOTAL"           => $qtservico,
            "VRCUSTOINFO"       => $vrtotal/$qtservico,
            "VRCUSTOIDEAL"      => $totalIdeal/$qtservico,
            "CDOPERULTATU"      => $cdoperador
        );
        
        $this->propostaFactory->alterarResultadoProposta($params);
    }
    /**
     * Retorna o código da ultima pk inserida na tabela de fct_relacformacaoprop 
    **/
    public function getCodeRelacFormacaoProp(){
        return $this->propostaFactory->getCodeRelacFormacaoProp();
    }
    /**
     * Busca as informações do item e manda salvar o novo valor
     * de acordo com a porcentagem que o serviço representa no orcamento
    **/
    public function updateItemOrcamento($nrseqorcamento, $nrorg, $totalAtual, $cdoperador){
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento, 
            "NRORG"          => $nrorg
        );
        $itens = $this->propostaFactory->getItensProposta($params);
        $vrorcamento = $itens[0]["VRORCAMENTO"];
        $diferenca   = $totalAtual - $vrorcamento;
        foreach ($itens as $item) {
            $paramsItens = array(
                "NRSEQORCAMENTO"  => $nrseqorcamento, 
                "NRORG"           => $nrorg,
                "NRITEMORCAMENTO" => $item["NRITEMORCAMENTO"],
                "ACRESCIMO"       => $diferenca * ($item["VRPERCITEMORCA"]/100),
                "CDOPERULTATU"    => $cdoperador
            );
            $this->propostaFactory->atualizarValorItem($paramsItens);
        }
    }
    /**
     * Alterar os valores no orcamento de acordo com o novo valor informado 
     * na alteração do valor do servico de uma proposta ja inserida
    **/
    public function atualizarItensAlteracaoResultado($nrseqorcamento, $nrorg, $cdoperador, $nrcompraiz, $qtservico, $vrunitServico){
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        
        $servicos = $this->propostaFactory->getInfoServicosProp($params);
        $totalOrcamento    = $qtservico * $vrunitServico;
        $totalOldOrcamento = 0;
        foreach($servicos as &$servico){
            $totalOldOrcamento = $totalOldOrcamento + $servico["VRTOTSERV"];
            if($servico["NRCOMPSERVICO"] != $nrcompraiz){
                $totalOrcamento = $totalOrcamento + $servico["VRTOTSERV"];
            } else {
                $servico["VRCUSTOINFORMADO"] = $vrunitServico;
            }
        }
        foreach($servicos as &$servico){
            $vrpercservico = ($servico["VRTOTSERV"] * 100)/ $totalOrcamento;
            $this->updateResultadoProposta($nrseqorcamento, $nrorg, $cdoperador, $servico["NRCOMPSERVICO"], $servico["VRCUSTOINFORMADO"], $vrpercservico);
        }
        
        $this->updateItemOrcamento($nrseqorcamento, $nrorg, $totalOrcamento, $cdoperador);
        $this->atualizarValorProposta($nrseqorcamento, $nrorg);
        $vroldpercmarkup = $this->propostaFactory->getVrMarkup($params)["VRPERCMARKUPAPLICADO"];
        $vrpercmarkup    = (($totalOrcamento/$this->getTotalSemMarkup($params)) - 1) * 100;
        
        $params= array_merge($params, array("VRPERCMARKUPAPLICADO" => $vrpercmarkup));
        $this->propostaFactory->atualizarMarkupAplicado($params);
        return $totalOrcamento;
    }
    /**
     * Altera os valores na tabela de resultado de acordo com 
     * a alteração do valor do servico de uma proposta ja inserida
    **/
    public function updateResultadoProposta($nrseqorcamento, $nrorg, $cdoperador, $nrcompraiz, $vrservico, $vrpercservico){
        $params = array(
            "CDOPERADOR"    => $cdoperador,
            "NRORG"         => $nrorg,
            "NRSEQORCAMENTO"=> $nrseqorcamento,
            "NRCOMPOSICAO"  => $nrcompraiz,
            "VRSERVICO"     => $vrservico,
            "VRPERCSERVICO" => $vrpercservico
        );
        $this->propostaFactory->updateResultadoProposta($params);
    }
}